package cn.rc100.rc_bd_map.bd_map_overlay;

import android.os.Bundle;

import com.baidu.mapapi.map.BaiduMap;
import com.baidu.mapapi.map.BitmapDescriptorFactory;
import com.baidu.mapapi.map.Marker;
import com.baidu.mapapi.map.MarkerOptions;
import com.baidu.mapapi.map.OverlayOptions;
import com.baidu.mapapi.map.Polyline;
import com.baidu.mapapi.search.poi.PoiResult;

import java.util.ArrayList;
import java.util.List;

import cn.rc100.rc_bd_map.R;


/**
 * 用于显示poi的overly
 */
public class PoiOverlay extends OverlayManager {

    private static final int MAX_POI_SIZE = 10;

    private PoiResult mPoiResult = null;

    private String keywordType = "公交";

    /**
     * 构造函数
     * 
     * @param baiduMap   该 PoiOverlay 引用的 BaiduMap 对象
     */
    public PoiOverlay(BaiduMap baiduMap,String keywordType) {
        super(baiduMap);
        this.keywordType = keywordType==null?this.keywordType:keywordType;
    }

    /**
     * 设置POI数据
     * 
     * @param poiResult    设置POI数据
     */
    public void setData(PoiResult poiResult) {
        this.mPoiResult = poiResult;
    }

    @Override
    public final List<OverlayOptions> getOverlayOptions() {
        if (mPoiResult == null || mPoiResult.getAllPoi() == null) {
            return null;
        }

        List<OverlayOptions> markerList = new ArrayList<>();
        int markerSize = 0;

        int iconId = R.mipmap.location;
        if("地铁".equals(this.keywordType)){
            iconId = R.mipmap.subway;
        }else if("公交".equals(this.keywordType)){
            iconId = R.mipmap.bus;
        }else if("医院".equals(this.keywordType)){
            iconId = R.mipmap.hospital;
        }else if("药店".equals(this.keywordType)){
            iconId = R.mipmap.drugstore;
        }else if("幼儿园".equals(this.keywordType)){
            iconId = R.mipmap.kindergarten;
        }else if("小学".equals(this.keywordType)){
            iconId = R.mipmap.primary_school;
        }else if("中学".equals(this.keywordType)){
            iconId = R.mipmap.middle_school;
        }else if("大学".equals(this.keywordType)){
            iconId = R.mipmap.university;
        }else if("商场".equals(this.keywordType)){
            iconId = R.mipmap.market;
        }else if("超市".equals(this.keywordType)){
            iconId = R.mipmap.supermarket;
        }else if("菜市场".equals(this.keywordType)){
            iconId = R.mipmap.vegetable_market;
        }else if("银行".equals(this.keywordType)){
            iconId = R.mipmap.bank;
        }else if("ATM".equals(this.keywordType)){
            iconId = R.mipmap.atm;
        }else if("餐厅".equals(this.keywordType)){
            iconId = R.mipmap.refectory;
        }else if("咖啡馆".equals(this.keywordType)){
            iconId = R.mipmap.coffee_shop;
        }else if("公园".equals(this.keywordType)){
            iconId = R.mipmap.park;
        }else if("电影院".equals(this.keywordType)){
            iconId = R.mipmap.cinema;
        }else if("健身房".equals(this.keywordType)){
            iconId = R.mipmap.gym;
        }else if("体育馆".equals(this.keywordType)){
            iconId = R.mipmap.sports_hall;
        }
        for (int i = 0; i < mPoiResult.getAllPoi().size() && markerSize < MAX_POI_SIZE; i++) {
            if (mPoiResult.getAllPoi().get(i).location == null) {
                continue;
            }

            markerSize++;
            Bundle bundle = new Bundle();
            bundle.putInt("index", i);

            markerList.add(new MarkerOptions()
                .icon(BitmapDescriptorFactory.fromResource(iconId))
                .extraInfo(bundle)
                .position(mPoiResult.getAllPoi().get(i).location));
            
        }

        return markerList;
    }

    /**
     * 获取该PoiOverlay的poi数据
     * 
     * @return     POI数据
     */
    public PoiResult getPoiResult() {
        return mPoiResult;
    }

    /**
     * 覆写此方法以改变默认点击行为
     * 
     * @param i    被点击的poi在
     *             {@link PoiResult#getAllPoi()} 中的索引
     * @return     true--事件已经处理，false--事件未处理
     */
    public boolean onPoiClick(int i) {
//        if (mPoiResult.getAllPoi() != null
//                && mPoiResult.getAllPoi().get(i) != null) {
//            Toast.makeText(BMapManager.getInstance().getContext(),
//                    mPoiResult.getAllPoi().get(i).name, Toast.LENGTH_LONG)
//                    .show();
//        }
        return false;
    }

    @Override
    public final boolean onMarkerClick(Marker marker) {
        if (!mOverlayList.contains(marker)) {
            return false;
        }

        if (marker.getExtraInfo() != null) {
            return onPoiClick(marker.getExtraInfo().getInt("index"));
        }

        return false;
    }

    @Override
    public boolean onPolylineClick(Polyline polyline) {
        return false;
    }
}
