package cn.rc100.rc_bd_map.bd_map_view;

import android.app.Activity;
import android.content.Context;
import android.view.View;
import android.widget.LinearLayout;

import com.baidu.mapapi.map.BaiduMap;
import com.baidu.mapapi.map.BaiduMapOptions;
import com.baidu.mapapi.map.BitmapDescriptor;
import com.baidu.mapapi.map.BitmapDescriptorFactory;
import com.baidu.mapapi.map.MapStatus;
import com.baidu.mapapi.map.MapStatusUpdate;
import com.baidu.mapapi.map.MapStatusUpdateFactory;
import com.baidu.mapapi.map.MapView;
import com.baidu.mapapi.map.MyLocationConfiguration;
import com.baidu.mapapi.map.MyLocationData;
import com.baidu.mapapi.model.LatLng;
import com.baidu.mapapi.search.core.PoiDetailInfo;
import com.baidu.mapapi.search.core.SearchResult;
import com.baidu.mapapi.search.poi.OnGetPoiSearchResultListener;
import com.baidu.mapapi.search.poi.PoiDetailResult;
import com.baidu.mapapi.search.poi.PoiDetailSearchOption;
import com.baidu.mapapi.search.poi.PoiDetailSearchResult;
import com.baidu.mapapi.search.poi.PoiIndoorResult;
import com.baidu.mapapi.search.poi.PoiNearbySearchOption;
import com.baidu.mapapi.search.poi.PoiResult;
import com.baidu.mapapi.search.poi.PoiSearch;
import com.baidu.mapapi.search.poi.PoiSortType;

import java.util.Map;

import cn.rc100.rc_bd_map.R;
import cn.rc100.rc_bd_map.bd_map_overlay.PoiOverlay;
import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.platform.PlatformView;

public class RcBdMapView extends LinearLayout implements PlatformView, MethodChannel.MethodCallHandler {
    private Context context;
    private Activity activity;
    private MapView mapView;

    private BaiduMap baiduMap;

    private PoiSearch mPoiSearch;

    private double currentLat = 0.0;
    private double currentLon = 0.0;
    private String keyword = "";

    public RcBdMapView(Activity activity, Context context, BinaryMessenger messenger, Map<String, Object> params) {
        super(context);
        this.context = context;
        this.activity = activity;

        BaiduMapOptions options = new BaiduMapOptions();
        //隐藏缩放控件
        options.zoomControlsEnabled(false);
        //禁用俯视图手势
        options.overlookingGesturesEnabled(false);
        //隐藏比例尺
        options.scaleControlEnabled(false);
        //禁用地图旋转
        options.rotateGesturesEnabled(false);

        mapView = new MapView(this.activity, options);
        //com.flutter_to_native_test_view_ 是更新数据的通信标识
        MethodChannel methodChannel = new MethodChannel(messenger, "BaiduMapPoi");
        methodChannel.setMethodCallHandler(this);
        if (baiduMap != null) {
            baiduMap.clear();
        }
        baiduMap = this.mapView.getMap();

        baiduMap.setMyLocationEnabled(true);

        //flutter 传递过来的参数
        if (params != null) {
//            currentLat = (double) ;
            currentLat = Double.parseDouble(params.get("latitude").toString());
            currentLon = Double.parseDouble(params.get("longitude").toString());
            if (params.get("keyword") != null) {
                keyword = (String) params.get("keyword");
            }
            initMap(currentLat, currentLon, keyword);
        }


    }

    public void initMap(double latitude, double longitude, String keyword) {
        BitmapDescriptor customMarker = BitmapDescriptorFactory.fromResource(R.mipmap.location);

        MyLocationConfiguration myLocationConfiguration = new MyLocationConfiguration(null, false, customMarker);

        MyLocationData locData = new MyLocationData.Builder()
                .latitude(latitude)
                .longitude(longitude).build();
        baiduMap.setMyLocationData(locData);
        baiduMap.setMyLocationConfiguration(myLocationConfiguration);

        //设定中心点坐标
        LatLng cenpt = new LatLng(latitude, longitude);
        //判断是否是详情页还是地图页面
        int zoomIndex = (keyword != null && !"".equals(keyword)) ? 18 : 16;
        //定义地图状态
        MapStatus mMapStatus = new MapStatus.Builder()
                .target(cenpt)
                .zoom(zoomIndex)
                .build();
        //定义MapStatusUpdate对象，以便描述地图状态将要发生的变化
        MapStatusUpdate mMapStatusUpdate = MapStatusUpdateFactory.newMapStatus(mMapStatus);
        //改变地图状态
        baiduMap.animateMapStatus(mMapStatusUpdate);
        if (keyword != null) {
            //poi检索
            mPoiSearch = PoiSearch.newInstance();
            mPoiSearch.setOnGetPoiSearchResultListener(poiListener);
            //搜索
            mPoiSearch.searchNearby(new PoiNearbySearchOption()
                    .location(cenpt)
                    .radius(1000)
                    .keyword(keyword)
                    .pageNum(0).sortType(PoiSortType.distance_from_near_to_far)
            );
        }
    }

    OnGetPoiSearchResultListener poiListener = new OnGetPoiSearchResultListener() {
        @Override
        public void onGetPoiResult(PoiResult poiResult) {
            if (poiResult.error == SearchResult.ERRORNO.NO_ERROR) {
                baiduMap.clear();

                //创建PoiOverlay对象
                PoiOverlay poiOverlay = new PoiOverlay(baiduMap, keyword);

                //设置Poi检索数据
                poiOverlay.setData(poiResult);

                //将poiOverlay添加至地图并缩放至合适级别
                poiOverlay.addToMap();
                poiOverlay.zoomToSpan(currentLat, currentLon);
            }
        }

        @Override
        public void onGetPoiDetailResult(PoiDetailSearchResult poiDetailSearchResult) {
            //PoiInfo 检索到的第一条信息
            PoiDetailInfo poi = poiDetailSearchResult.getPoiDetailInfoList().get(0);
            //通过第一条检索信息对应的uid发起详细信息检索
            mPoiSearch.searchPoiDetail((new PoiDetailSearchOption())
                    .poiUids(poi.getUid())); // uid的集合，最多可以传入10个uid，多个uid之间用英文逗号分隔。
        }

        @Override
        public void onGetPoiIndoorResult(PoiIndoorResult poiIndoorResult) {

        }

        //废弃
        @Override
        public void onGetPoiDetailResult(PoiDetailResult poiDetailResult) {

        }
    };

    @Override
    public View getView() {
        return mapView;
    }

    @Override
    public void dispose() {

    }

    @Override
    public void onMethodCall(MethodCall methodCall, MethodChannel.Result result) {
        Map<String, Object> params = (Map<String, Object>) methodCall.arguments;
        if (params != null) {
            currentLat = Double.parseDouble(params.get("latitude").toString());
            currentLon = Double.parseDouble(params.get("longitude").toString());
            keyword = params.get("keyword").toString();
            initMap(currentLat, currentLon, keyword);
        }
    }
}
