//
//  BaiduMapView.m
//  Pods-Runner
//
//  Created by 荣城 on 2020/1/18.
//

#import "BaiduMapView.h"

@interface BaiduMapView ()<BMKMapViewDelegate>
@property (nonatomic,strong) BMKMapView *mapView;
@end


@implementation BaiduMapView
    int64_t _viewId;
    NSString *keyword;
    BMKMapView *_mapView;
    FlutterMethodChannel *_channel;
    NSString *latitude;
    NSString *longitude;
    NSString *name;


-(instancetype)initWithWithFrame:(CGRect)frame viewIdentifier:(int64_t)viewId arguments:(id)args binaryMessenger:(NSObject<FlutterBinaryMessenger> *)messenger{
    if([super init]){
        if(frame.size.width==0){
            frame = CGRectMake(frame.origin.x, frame.origin.y, [UIScreen mainScreen].bounds.size.width, 22);
        }
        if(args!=nil){
            NSDictionary *dic = args;
            latitude = dic[@"latitude"];
            longitude = dic[@"longitude"];
            keyword = dic[@"keyword"];
            name = dic[@"name"];
            _mapView = [[BMKMapView alloc]initWithFrame:self.view.frame];
            _mapView.delegate = self;
            //显示定位图层
            _mapView.showsUserLocation = YES;
            //是否允许拖动
            _mapView.scrollEnabled = NO;
            
            BMKMapStatus * mapStatus = [[BMKMapStatus alloc] init];
            mapStatus.fLevel = 17;
            //设置屏幕中心点坐标（超过屏幕外无效）
            mapStatus.targetGeoPt = CLLocationCoordinate2DMake([latitude doubleValue],[longitude doubleValue]);
            [_mapView setMapStatus:mapStatus];
            
            _mapView.userTrackingMode = BMKUserTrackingModeNone;
            self.initMap;
            
        }
        
        _channel = [FlutterMethodChannel methodChannelWithName:@"BaiduMapPoi" binaryMessenger:messenger];
        [_channel setMethodCallHandler:^(FlutterMethodCall * _Nonnull call, FlutterResult  _Nonnull result) {
            NSDictionary *dic = call.arguments;
            if(dic!=nil){
                latitude = dic[@"latitude"];
                longitude = dic[@"longitude"];
                keyword = dic[@"keyword"];
                self.initMap;
            }
        }];

        _viewId = viewId;
    }
    return self;
}

-(void)initMap{
    if(keyword != [NSNull null] && keyword != nil){
        
        //根据坐标设置中心点
        BMKUserLocation *location = [[BMKUserLocation alloc] init];
        location.location = [[CLLocation alloc]initWithLatitude:[latitude doubleValue] longitude:[longitude doubleValue]];
        if(name!= [NSNull null]){
           location.title = name;
        }else{
           location.title = @"楼盘信息";
        }
        //更新中心点位置
        [_mapView updateLocationData:location];

        BMKLocationViewDisplayParam * param = [[BMKLocationViewDisplayParam alloc] init];
        param.locationViewImage = [UIImage imageNamed:@"LocationMax"];
        param.locationViewHierarchy = LOCATION_VIEW_HIERARCHY_TOP;
        [_mapView updateLocationViewWithParam:param];
        
        BMKPoiSearch *poiSearch = [[BMKPoiSearch alloc] init];
        poiSearch.delegate = self;
        BMKPOINearbySearchOption *nearbyOption = [[BMKPOINearbySearchOption alloc] init];
        nearbyOption.keywords = @[keyword];
        nearbyOption.location = CLLocationCoordinate2DMake([latitude doubleValue],[longitude doubleValue]);
        nearbyOption.radius = 1000;
        nearbyOption.tags = @[keyword];
        nearbyOption.isRadiusLimit = YES;
        nearbyOption.pageIndex = 0;
        nearbyOption.pageSize = 10;
        BOOL flag = [poiSearch poiSearchNearBy:nearbyOption];
        if(flag){
            NSLog(@"检索成功");
        }else{
            NSLog(@"检索失败");
        }
    }else{
        BMKPointAnnotation *annotation = [[BMKPointAnnotation alloc] init];
        annotation.coordinate =CLLocationCoordinate2DMake([latitude doubleValue],[longitude doubleValue]);
        annotation.title = name;
        [_mapView addAnnotation:annotation];
        [_mapView selectAnnotation:annotation animated:YES];
    }
}


#pragma mark - BMKPoiSearchDelegate
- (void)onGetPoiResult:(BMKPoiSearch *)searcher result:(BMKPOISearchResult *)poiResult errorCode:(BMKSearchErrorCode)error {
    
    if(keyword == nil || keyword == [NSNull null]){
        return;
    }
    /**
    移除一组标注
    @param annotations 要移除的标注数组
    */
    [_mapView removeAnnotations:_mapView.annotations];
    //BMKSearchErrorCode错误码，BMK_SEARCH_NO_ERROR：检索结果正常返回
    if (error == BMK_SEARCH_NO_ERROR) {
        NSMutableArray *annotations = [NSMutableArray array];
        for (NSUInteger i = 0; i < poiResult.poiInfoList.count; i ++) {
            //POI信息类的实例
            BMKPoiInfo *POIInfo = poiResult.poiInfoList[i];
            //初始化标注类BMKPointAnnotation的实例
            BMKPointAnnotation *annotation = [[BMKPointAnnotation alloc]init];
            //设置标注的经纬度坐标
            annotation.coordinate = POIInfo.pt;
            //设置标注的标题
            annotation.title = POIInfo.name;
            [annotations addObject:annotation];
        }
        //将一组标注添加到当前地图View中
        [_mapView addAnnotations:annotations];
    }
}

#pragma mark - BMKMapViewDelegate
- (BMKAnnotationView *)mapView:(BMKMapView *)mapView viewForAnnotation:(id<BMKAnnotation>)annotation{
    if ([annotation isKindOfClass:[BMKPointAnnotation class]]) {
        static NSString *pointReuseIndentifier = @"pointReuseIndentifier";
        BMKPinAnnotationView*annotationView = (BMKPinAnnotationView *)[mapView dequeueReusableAnnotationViewWithIdentifier:pointReuseIndentifier];
        if (annotationView == nil) {
            annotationView = [[BMKPinAnnotationView alloc] initWithAnnotation:annotation reuseIdentifier:pointReuseIndentifier];
        }
        annotationView.pinColor = BMKPinAnnotationColorRed;
        annotationView.canShowCallout= YES;      //设置气泡可以弹出，默认为NO
        annotationView.animatesDrop=NO;         //设置标注动画显示，默认为NO
        annotationView.draggable = NO;          //设置标注可以拖动，默认为NO
        if(keyword!=[NSNull null]&&keyword!=nil){
            annotationView.image = [UIImage imageNamed:keyword];
        }else{
            annotationView.image = [UIImage imageNamed:@"Location"];
        }
        return annotationView;
    }
    return nil;
}

#pragma mark - View life cycle
-(void) viewDidLoad{
    [super viewDidLoad];
}

-(void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:NO];
    //当mapView即将被显示的时候调用，恢复之前存储的mapView状态
    [_mapView viewWillAppear];
}

-(void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:NO];
    //当mapView即将被隐藏的时候调用，存储当前mapView的状态
    [_mapView viewWillDisappear];
}

-(nonnull UIView *)view{
    return _mapView;
}
@end

