import 'dart:async';
import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:flutter/services.dart';

class BaiduLocation{
  final double latitude;
  final double longitude;
  final String address;
  final String city;
  final String cityCode;
  final String adCode;


  BaiduLocation({this.latitude, this.longitude,this.address,
    this.city,this.cityCode,this.adCode});

  factory BaiduLocation.fromMap(dynamic value){
    return new BaiduLocation(
        latitude: value['latitude'],
        longitude:value['longitude'],
        address: value['address'],
        city: value['city'],
        cityCode: value['cityCode'],
        adCode:value['adCode']
    );
  }


  Map getMap() {
    return {
      "latitude": latitude,
      "longitude":longitude,
      "address":address,
      "city": city,
      "cityCode": cityCode,
      "adCode":adCode,
    };
  }
}

class RcBdMap {
  static const MethodChannel _channel =
      const MethodChannel('rc_bd_map');

  static Future<String> get platformVersion async {
    final String version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }

  static const MethodChannel _bdMapChannel = const MethodChannel("rc_bd_map_location");

  ///初始化地图
  static Future<bool> initMap(key) async{
    bool result;
    if(Platform.isIOS && key != null) {
      result = await _bdMapChannel.invokeMethod(
          "initLocationByKey", key);
    }else if(Platform.isAndroid ){
      result = true;
    }else{
      result = false;
    }
    if(!result&&!Platform.isAndroid){
      print("地图初始化失败！请坚持百度地图应用是否正确！");
    }
    return result;
  }

  /// 获取经纬度
  static Future<BaiduLocation> getCurrentLocation() async{
    final Map result = await _bdMapChannel.invokeMethod('getCurrentLocation');
    return new BaiduLocation.fromMap(result);
  }

  static const MethodChannel _bdMapPoiChannel = const MethodChannel("BaiduMapPoi");

  ///根据关键字在地图上展示相应的poi检索内容
  static Future<String> checkMapType(double latitude,double longitude,String keyword) async{
    print(keyword);
    if (keyword == null || keyword == "") {
      return "";
    }
    Map<String, Object> map = {
      "latitude": latitude,
      "longitude": longitude,
      "keyword": keyword == null ? keyword : keyword
    };
    String result = await _bdMapPoiChannel.invokeMethod("BaiduMapPoi", map);
  }

  ///获取地图展示
  static Widget getMap(double latitude,double longitude,String name){
    if(latitude==null || longitude == null){
      return Container(

      );
    }
    if(Platform.isAndroid){
      return  AndroidView(
        //禁止拖动
        viewType: 'BaiduMapView',
        creationParams: {
          "latitude": latitude,
          "longitude": longitude,
          "name": name
        },
        //参数的编码方式
        creationParamsCodec: StandardMessageCodec(),
      );
    }else if(Platform.isIOS){
      return UiKitView(
        viewType: 'BaiduMapView',
        creationParams: {
          "latitude": latitude,
          "longitude": longitude,
          "name":"和谐世纪"
        },
        //参数的编码方式
        creationParamsCodec: StandardMessageCodec(),
      );
    }
  }

}
